"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const llparse_builder_1 = require("llparse-builder");
exports.code = llparse_builder_1.code;
exports.node = llparse_builder_1.node;
exports.Span = llparse_builder_1.Span;
const builder = require("llparse-builder");
exports.builder = builder;
const compiler_1 = require("./compiler");
/**
 * LLParse graph builder and compiler.
 */
class LLParse extends llparse_builder_1.Builder {
    /**
     * The prefix controls the names of methods and state struct in generated
     * public C headers:
     *
     * ```c
     * // state struct
     * struct PREFIX_t {
     *   ...
     * }
     *
     * int PREFIX_init(PREFIX_t* state);
     * int PREFIX_execute(PREFIX_t* state, const char* p, const char* endp);
     * ```
     *
     * @param prefix  Prefix to be used when generating public API.
     */
    constructor(prefix = 'llparse') {
        super();
        this.prefix = prefix;
    }
    /**
     * Compile LLParse graph to the bitcode and C headers
     *
     * @param root  Root node of the parse graph (see `.node()`)
     * @param options Compiler options.
     */
    build(root, options = {}) {
        const c = new compiler_1.Compiler(this.prefix, options);
        return c.compile(root, this.properties);
    }
}
exports.LLParse = LLParse;
//# sourceMappingURL=api.js.map