/// <reference types="node" />
import { Builder, node as apiNode, Property as APIProperty } from 'llparse-builder';
import * as builder from 'llparse-builder';
import { ITranslatorLazyOptions } from '../translator';
export { builder, Builder };
export interface ICompilerOptions {
    /**
     * Debug method name
     *
     * The method must have following signature:
     *
     * ```c
     * void debug(llparse_t* state, const char* p, const char* endp,
     *            const char* msg);
     * ```
     *
     * Where `llparse_t` is a parser state type.
     */
    readonly debug?: string;
    /** Translator options */
    readonly translator?: ITranslatorLazyOptions;
    /** What guard define to use in `#ifndef` in C headers */
    readonly headerGuard?: string;
}
/** Build artifacts */
export interface ICompilerResult {
    /** Bitcode output */
    readonly bitcode: Buffer;
    /** C header */
    readonly header: string;
}
export declare class Compiler {
    private readonly prefix;
    private readonly options;
    constructor(prefix: string, options: ICompilerOptions);
    compile(apiRoot: apiNode.Node, properties: ReadonlyArray<APIProperty>): ICompilerResult;
    createBuilder(): Builder;
}
