"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debugAPI = require("debug");
const llparse_builder_1 = require("llparse-builder");
exports.Builder = llparse_builder_1.Builder;
const builder = require("llparse-builder");
exports.builder = builder;
const compilation_1 = require("../compilation");
const translator_1 = require("../translator");
const execute_builder_1 = require("./execute-builder");
const init_builder_1 = require("./init-builder");
const debug = debugAPI('llparse:compiler');
class Compiler {
    constructor(prefix, options) {
        this.prefix = prefix;
        this.options = options;
    }
    compile(apiRoot, properties) {
        // Check if loops are present
        debug('checking loops');
        const lc = new llparse_builder_1.LoopChecker();
        lc.check(apiRoot);
        // Allocate spans
        debug('allocating spans');
        const sa = new llparse_builder_1.SpanAllocator();
        const spans = sa.allocate(apiRoot);
        // Translate to compiler nodes
        debug('translating nodes');
        const t = new translator_1.Translator(this.prefix, this.options.translator || {}, spans);
        const root = t.translate(apiRoot);
        // Compile to bitcode
        const compilation = new compilation_1.Compilation(this.prefix, root, properties, t.spans, this.options);
        debug('building root');
        const initFn = root.build(compilation);
        compilation.addResumptionTarget(initFn);
        debug('building init');
        const ib = new init_builder_1.InitBuilder();
        ib.build(compilation, initFn);
        debug('building execute');
        const eb = new execute_builder_1.ExecuteBuilder();
        eb.build(compilation, t.spans);
        debug('building bitcode');
        const bitcode = compilation.buildBitcode(initFn);
        debug('building header');
        const header = compilation.buildHeader();
        debug('done');
        return { bitcode, header };
    }
    // Convenience
    createBuilder() {
        return new llparse_builder_1.Builder();
    }
}
exports.Compiler = Compiler;
//# sourceMappingURL=index.js.map