"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const base_1 = require("./base");
const error_1 = require("./error");
class Invoke extends base_1.Node {
    constructor(id, code) {
        super(id);
        this.code = code;
        this.edges = [];
    }
    addEdge(code, node) {
        this.edges.push({ code, node });
    }
    doBuild(bb, pos) {
        const ctx = this.compilation;
        const code = this.code.build(ctx);
        const args = [
            ctx.stateArg(bb),
            pos.current,
            ctx.endPosArg(bb),
        ];
        if (this.code.signature === 'value') {
            args.push(ctx.matchArg(bb));
        }
        else {
            assert.strictEqual(this.code.signature, 'match', 'Passing `span` callback to `invoke` is not allowed');
        }
        const call = bb.call(code, args);
        const keys = this.edges.map((edge) => edge.code);
        const s = ctx.switch(bb, call, keys, {
            cases: this.edges.map((edge) => {
                return edge.node instanceof error_1.Error ? 'unlikely' : 'likely';
            }),
            otherwise: this.otherwise instanceof error_1.Error ? 'unlikely' : 'likely',
        });
        s.cases.forEach((childBB, index) => {
            const edge = this.edges[index];
            this.tailTo(childBB, {
                noAdvance: true,
                node: edge.node,
                value: undefined,
            }, pos);
        });
        this.tailTo(s.otherwise, this.otherwise, pos);
    }
}
exports.Invoke = Invoke;
//# sourceMappingURL=invoke.js.map