"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const error_1 = require("./error");
const match_1 = require("./match");
class Single extends match_1.Match {
    constructor(id) {
        super(id);
        this.edges = [];
    }
    addEdge(edge) {
        this.edges.push(edge);
    }
    doBuild(bb, pos) {
        bb = this.prologue(bb, pos);
        const ctx = this.compilation;
        // Load the character
        let current = bb.load(pos.current);
        // Transform the character
        current = this.transform.build(ctx, bb, current);
        // Mark error branches as unlikely
        const cases = this.edges.map((edge) => {
            if (edge.node instanceof error_1.Error) {
                return 'unlikely';
            }
            else {
                return 'likely';
            }
        });
        const weight = {
            cases,
            otherwise: this.otherwise.node instanceof error_1.Error ?
                'unlikely' : 'likely',
        };
        const keys = this.edges.map((edge) => edge.key);
        const s = ctx.switch(bb, current, keys, weight);
        s.cases.forEach((caseBB, i) => {
            const edge = this.edges[i];
            this.tailTo(caseBB, edge, pos);
        });
        this.tailTo(s.otherwise, this.otherwise, pos);
    }
}
exports.Single = Single;
//# sourceMappingURL=single.js.map